#include "main.h"
#include "stm32f4xx_hal.h"
#include "stm32f4xx_nucleo.h"

#include "serialComm.h"

#include "FreeRTOS.h"
#include "task.h"
#include "timers.h"
#include "queue.h"
#include "semphr.h"
#include "event_groups.h"

#include <stdarg.h>
#include <string.h>

void SystemClock_Config(void);

/* Tasks -----------------------------------------------------------*/
void LedBlinkTask(void const * argument);
void ButtonHandler(void const * argument);
void SerialPrintTask(void const * argument);

volatile uint32_t counter = 0;

SemaphoreHandle_t btnSemaphore;

int main(void)
{
	HAL_Init();
	SystemClock_Config();

	BSP_LED_Init(LED2);
	BSP_PB_Init(BUTTON_USER, BUTTON_MODE_EXTI);

	btnSemaphore = xSemaphoreCreateBinary();

	xTaskCreate((void *)vSerialCommTask, "SERIALINIT", configMINIMAL_STACK_SIZE, NULL, tskIDLE_PRIORITY, NULL);
	xTaskCreate((void *)LedBlinkTask, "BLINK", configMINIMAL_STACK_SIZE, NULL, tskIDLE_PRIORITY, NULL);
	xTaskCreate((void *)ButtonHandler, "BUTTON", configMINIMAL_STACK_SIZE, NULL, tskIDLE_PRIORITY, NULL);

	xTaskCreate((void *)SerialPrintTask, "Serial0", configMINIMAL_STACK_SIZE, "Serial0", tskIDLE_PRIORITY, NULL);
	xTaskCreate((void *)SerialPrintTask, "Serial1", configMINIMAL_STACK_SIZE, "Serial1", tskIDLE_PRIORITY, NULL);
	xTaskCreate((void *)SerialPrintTask, "Serial2", configMINIMAL_STACK_SIZE, "Serial2", tskIDLE_PRIORITY, NULL);
	xTaskCreate((void *)SerialPrintTask, "Serial3", configMINIMAL_STACK_SIZE, "Serial3", tskIDLE_PRIORITY, NULL);
	xTaskCreate((void *)SerialPrintTask, "Serial4", configMINIMAL_STACK_SIZE, "Serial4", tskIDLE_PRIORITY, NULL);
	xTaskCreate((void *)SerialPrintTask, "Serial5", configMINIMAL_STACK_SIZE, "Serial5", tskIDLE_PRIORITY, NULL);
	xTaskCreate((void *)SerialPrintTask, "Serial6", configMINIMAL_STACK_SIZE, "Serial6", tskIDLE_PRIORITY, NULL);
	xTaskCreate((void *)SerialPrintTask, "Serial7", configMINIMAL_STACK_SIZE, "Serial7", tskIDLE_PRIORITY, NULL);
	xTaskCreate((void *)SerialPrintTask, "Serial8", configMINIMAL_STACK_SIZE, "Serial8", tskIDLE_PRIORITY, NULL);
	xTaskCreate((void *)SerialPrintTask, "Serial9", configMINIMAL_STACK_SIZE, "Serial9", tskIDLE_PRIORITY, NULL);

	vTaskStartScheduler();

	while (1)
	{
		BSP_LED_Toggle(LED2);
		HAL_Delay(50);
	}

}

void LedBlinkTask(void const * argument)
{
	while(1)
	{
		BSP_LED_Toggle(LED2);
		vTaskDelay(1000);
	}

	vTaskDelete(NULL);
}

void SerialPrintTask(void const * argument)
{
	char *pcStringToPrint;
	pcStringToPrint = ( char * ) argument;

	while(1)
	{
		counter++;
		//serialOut((uint8_t *) &pcStringToPrint, 7);
		printOut("%s: %d\r\n", pcStringToPrint, counter);
		vTaskDelay(250);
	}

	vTaskDelete(NULL);
}

void ButtonHandler(void const * argument)
{
	while(1)
	{
		if (btnSemaphore != NULL)
		{
			if (xSemaphoreTake(btnSemaphore, portMAX_DELAY) == pdTRUE)
			{
				printOut("Button Pressed!\r\n");
			}
		}
	}

	vTaskDelete(NULL);
}

void HAL_GPIO_EXTI_Callback(uint16_t GPIO_Pin)
{
	if(GPIO_Pin == USER_BUTTON_PIN)
	{
		BaseType_t xHigherPriorityTaskWoken;
		xHigherPriorityTaskWoken = pdFALSE;
		xSemaphoreGiveFromISR(btnSemaphore, &xHigherPriorityTaskWoken);
		portYIELD_FROM_ISR(xHigherPriorityTaskWoken);
	}
}

void SystemClock_Config(void)
{

	RCC_OscInitTypeDef RCC_OscInitStruct;
	RCC_ClkInitTypeDef RCC_ClkInitStruct;

	__HAL_RCC_PWR_CLK_ENABLE();

	__HAL_PWR_VOLTAGESCALING_CONFIG(PWR_REGULATOR_VOLTAGE_SCALE2);

	RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_HSI;
	RCC_OscInitStruct.HSIState = RCC_HSI_ON;
	RCC_OscInitStruct.HSICalibrationValue = 16;
	RCC_OscInitStruct.PLL.PLLState = RCC_PLL_ON;
	RCC_OscInitStruct.PLL.PLLSource = RCC_PLLSOURCE_HSI;
	RCC_OscInitStruct.PLL.PLLM = 16;
	RCC_OscInitStruct.PLL.PLLN = 336;
	RCC_OscInitStruct.PLL.PLLP = RCC_PLLP_DIV4;
	RCC_OscInitStruct.PLL.PLLQ = 7;
	if (HAL_RCC_OscConfig(&RCC_OscInitStruct) != HAL_OK)
	{
		_Error_Handler(__FILE__, __LINE__);
	}

	RCC_ClkInitStruct.ClockType = RCC_CLOCKTYPE_HCLK|RCC_CLOCKTYPE_SYSCLK
			|RCC_CLOCKTYPE_PCLK1|RCC_CLOCKTYPE_PCLK2;
	RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_PLLCLK;
	RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
	RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV2;
	RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV1;

	if (HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_2) != HAL_OK)
	{
		_Error_Handler(__FILE__, __LINE__);
	}

	HAL_SYSTICK_Config(HAL_RCC_GetHCLKFreq()/1000);
	HAL_SYSTICK_CLKSourceConfig(SYSTICK_CLKSOURCE_HCLK);
	HAL_NVIC_SetPriority(SysTick_IRQn, 15, 0);
}

void _Error_Handler(char *file, int line)
{
	while(1)
	{
	}
}

#ifdef  USE_FULL_ASSERT
void assert_failed(uint8_t* file, uint32_t line)
{ 
}
#endif
